@@ -76,15 +76,6 @@ static const char introspection_xml[] =
   "   <arg type='s' name='uri' direction='out'/>"
   "   <arg type='s' name='color' direction='out'/>"
   "  </method>"
-  "  <signal name='FormAuthDataSaveConfirmationRequired'>"
-  "   <arg type='u' name='request_id' direction='out'/>"
-  "   <arg type='t' name='page_id' direction='out'/>"
-  "   <arg type='s' name='hostname' direction='out'/>"
-  "   <arg type='s' name='username' direction='out'/>"
-  "  </signal>"
-  "  <signal name='RemoveItemFromOverview'>"
-  "   <arg type='s' name='url' direction='out'/>"
-  "  </signal>"
   "  <method name='FormAuthDataSaveConfirmationResponse'>"
   "   <arg type='u' name='request_id' direction='in'/>"
   "   <arg type='b' name='should_store' direction='in'/>"
@@ -107,9 +98,6 @@ static const char introspection_xml[] =
   "   <arg type='s' name='host' direction='in'/>"
   "  </method>"
   "  <method name='HistoryClear'/>"
-  "  <signal name='AllowTLSCertificate'>"
-  "   <arg type='t' name='page_id' direction='out'/>"
-  "  </signal>"
   " </interface>"
   "</node>";
 
@@ -241,11 +229,13 @@ request_decision_on_storing (EphyEmbedFormAuth *form_auth)
   char *username_field_value = NULL;
   guint request_id;
   SoupURI *uri;
-  GError *error = NULL;
   WebKitDOMNode *username_node;
-  EphyWebExtension *extension = ephy_web_extension_get ();
+  WebKitDOMDOMWindow *dom_window;
+  GVariant *variant;
+  gchar *message;
 
-  if (!extension->priv->dbus_connection) {
+  dom_window = webkit_dom_document_get_default_view (ephy_embed_form_auth_get_owner_document (form_auth));
+  if (!dom_window) {
     g_object_unref (form_auth);
     return;
   }
@@ -256,53 +246,28 @@ request_decision_on_storing (EphyEmbedFormAuth *form_auth)
   if (username_node)
     g_object_get (username_node, "value", &username_field_value, NULL);
 
-  g_dbus_connection_emit_signal (extension->priv->dbus_connection,
-                                 NULL,
-                                 EPHY_WEB_EXTENSION_OBJECT_PATH,
-                                 EPHY_WEB_EXTENSION_INTERFACE,
-                                 "FormAuthDataSaveConfirmationRequired",
-                                 g_variant_new ("(utss)",
-                                                request_id,
-                                                ephy_embed_form_auth_get_page_id (form_auth),
-                                                uri ? uri->host : "",
-                                                username_field_value ? username_field_value : ""),
-                                 &error);
-  if (error) {
-    g_warning ("Error emitting signal FormAuthDataSaveConfirmationRequired: %s\n", error->message);
-    g_error_free (error);
-  } else {
-    g_hash_table_insert (ephy_web_extension_get_form_auth_data_save_requests (extension),
+  variant = g_variant_new ("(utss)",
+                           request_id,
+                           ephy_embed_form_auth_get_page_id (form_auth),
+                           uri ? uri->host : "",
+                           username_field_value ? username_field_value : "");
+  g_free (username_field_value);
+
+  message = g_variant_print (variant, FALSE);
+  g_variant_unref (variant);
+
+  if (webkit_dom_dom_window_webkit_message_handlers_post_message (dom_window, "formAuthData", message)) {
+    g_hash_table_insert (ephy_web_extension_get_form_auth_data_save_requests (ephy_web_extension_get ()),
                          GINT_TO_POINTER (request_id),
                          g_object_ref (form_auth));
+  } else {
+    g_warning ("Error sending formAuthData message");
   }
 
-  g_free (username_field_value);
+  g_free (message);
   g_object_unref (form_auth);
 }
 
-static void
-overview_item_removed (EphyWebOverview *overview,
-                       const char *url,
-                       EphyWebExtension *extension)
-{
-  GError *error = NULL;
-
-  if (!extension->priv->dbus_connection)
-    return;
-
-  g_dbus_connection_emit_signal (extension->priv->dbus_connection,
-                                 NULL,
-                                 EPHY_WEB_EXTENSION_OBJECT_PATH,
-                                 EPHY_WEB_EXTENSION_INTERFACE,
-                                 "RemoveItemFromOverview",
-                                 g_variant_new ("(s)", url),
-                                 &error);
-  if (error) {
-    g_debug ("Error emitting signal RemoveItemFromOverview: %s\n", error->message);
-    g_error_free (error);
-  }
-}
-
 static void
 should_store_cb (const char *username,
                  const char *password,
@@ -1016,6 +981,19 @@ web_page_document_loaded (WebKitWebPage *web_page,
   g_object_unref(forms);
 }
 
+static void
+web_page_uri_changed (WebKitWebPage *web_page,
+                      GParamSpec *param_spec,
+                      EphyWebExtension *extension)
+{
+  EphyWebOverview *overview = NULL;
+
+  if (g_strcmp0 (webkit_web_page_get_uri (web_page), "ephy-about:overview") == 0)
+    overview = ephy_web_overview_new (web_page, extension->priv->overview_model);
+
+  g_object_set_data_full (G_OBJECT (web_page), "ephy-web-overview", overview, g_object_unref);
+}
+
 static void
 ephy_web_extension_emit_page_created (EphyWebExtension *extension,
                                       guint64 page_id)
@@ -1081,6 +1059,9 @@ ephy_web_extension_page_created_cb (EphyWebExtension *extension,
   g_signal_connect (web_page, "document-loaded",
                     G_CALLBACK (web_page_document_loaded),
                     extension);
+  g_signal_connect (web_page, "notify::uri",
+                    G_CALLBACK (web_page_uri_changed),
+                    extension);
 }
 
 static WebKitWebPage *
@@ -1248,137 +1229,6 @@ static const GDBusInterfaceVTable interface_vtable = {
   NULL
 };
 
-typedef struct {
-  EphyWebExtension *extension;
-  guint64 page_id;
-} AllowTLSCertificateData;
-
-static JSValueRef
-allow_tls_certificate_cb (JSContextRef context,
-                          JSObjectRef function,
-                          JSObjectRef this_object,
-                          size_t argument_count,
-                          const JSValueRef arguments[],
-                          JSValueRef *exception)
-{
-  AllowTLSCertificateData *data;
-  EphyWebExtension *extension;
-  GError *error = NULL;
-
-  data = (AllowTLSCertificateData *)JSObjectGetPrivate (this_object);
-  extension = data->extension;
-
-  if (!extension->priv->dbus_connection)
-    return JSValueMakeUndefined (context);
-
-  g_dbus_connection_emit_signal (extension->priv->dbus_connection,
-                                 NULL,
-                                 EPHY_WEB_EXTENSION_OBJECT_PATH,
-                                 EPHY_WEB_EXTENSION_INTERFACE,
-                                 "AllowTLSCertificate",
-                                 g_variant_new ("(t)", data->page_id),
-                                 &error);
-
-  if (error) {
-    g_warning ("Error emitting signal AllowTLSCertificate: %s\n", error->message);
-    g_error_free (error);
-  }
-
-  return JSValueMakeUndefined (context);
-}
-
-static const JSStaticFunction tls_certificate_error_page_static_funcs[] =
-{
-  { "allowException", allow_tls_certificate_cb, kJSPropertyAttributeReadOnly | kJSPropertyAttributeDontDelete },
-  { NULL, NULL, 0 }
-};
-
-static void
-tls_certificate_error_page_finalize (JSObjectRef object)
-{
-  g_slice_free (AllowTLSCertificateData, JSObjectGetPrivate (object));
-}
-
-static void
-prepare_certificate_exception_js (WebKitScriptWorld *world,
-                                  WebKitWebPage *web_page,
-                                  WebKitFrame *frame,
-                                  EphyWebExtension *extension)
-{
-  JSGlobalContextRef context;
-  JSObjectRef global_object;
-  JSClassDefinition class_def;
-  JSClassRef class;
-  JSObjectRef class_object;
-  JSStringRef str;
-  AllowTLSCertificateData *data;
-
-  context = webkit_frame_get_javascript_context_for_script_world (frame, world);
-  global_object = JSContextGetGlobalObject (context);
-
-  class_def = kJSClassDefinitionEmpty;
-  class_def.className = "EpiphanyTLSCertificateErrorPage";
-  class_def.staticFunctions = tls_certificate_error_page_static_funcs;
-  class_def.finalize = tls_certificate_error_page_finalize;
-
-  data = g_slice_alloc (sizeof (AllowTLSCertificateData));
-  data->extension = extension;
-  data->page_id = webkit_web_page_get_id (web_page);
-
-  class = JSClassCreate (&class_def);
-  class_object = JSObjectMake (context, class, data);
-  str = JSStringCreateWithUTF8CString ("EpiphanyTLSCertificateErrorPage");
-  JSObjectSetProperty (context, global_object, str, class_object, kJSPropertyAttributeNone, NULL);
-
-  JSClassRelease (class);
-  JSStringRelease (str);
-}
-
-static void
-prepare_overview (WebKitScriptWorld *world,
-                  WebKitWebPage *web_page,
-                  WebKitFrame *frame,
-                  EphyWebExtension *extension)
-{
-  EphyWebOverview *overview;
-  JSGlobalContextRef context;
-
-  overview = ephy_web_overview_new (web_page, extension->priv->overview_model);
-  g_signal_connect (overview, "item-removed",
-                    G_CALLBACK (overview_item_removed),
-                    extension);
-  context = webkit_frame_get_javascript_context_for_script_world (frame, world);
-  ephy_web_overview_init_js (overview, context);
-}
-
-static void
-window_object_cleared_cb (WebKitScriptWorld *world,
-                          WebKitWebPage     *web_page,
-                          WebKitFrame       *frame,
-                          EphyWebExtension  *extension)
-{
-  WebKitDOMDocument *dom_document;
-  char *dom_url;
-
-  if (g_strcmp0 (webkit_web_page_get_uri (web_page), "ephy-about:overview") == 0) {
-    prepare_overview (world, web_page, frame, extension);
-    return;
-  }
-
-  dom_document = webkit_web_page_get_dom_document (web_page);
-  dom_url = webkit_dom_document_get_url (dom_document);
-
-  /* If webkit_web_page_get_uri is not about:blank and webkit_dom_document_get_url is
-   * about:blank, we are likely loading alternate content, so it's safe to make the
-   * certificate exception js available. This is needed by the TLS error page. */
-  if (g_strcmp0 (webkit_web_page_get_uri (web_page), "about:blank") != 0 &&
-      g_strcmp0 (dom_url, "about:blank") == 0) {
-    prepare_certificate_exception_js (world, web_page, frame, extension);
-  }
-
-  g_free (dom_url);
-}
-
 static void
 ephy_web_extension_dispose (GObject *object)
 {
@@ -1426,11 +1276,6 @@ ephy_web_extension_init (EphyWebExtension *extension)
 {
   extension->priv = G_TYPE_INSTANCE_GET_PRIVATE (extension, EPHY_TYPE_WEB_EXTENSION, EphyWebExtensionPrivate);
   extension->priv->overview_model = ephy_web_overview_model_new ();
-
-  g_signal_connect (webkit_script_world_get_default (),
-                    "window-object-cleared",
-                    G_CALLBACK (window_object_cleared_cb),
-                    extension);
 }
 
 static gpointer
