@@ -359,6 +359,7 @@ ephy_web_dom_utils_find_form_auth_elements (WebKitDOMHTMLFormElement *form,
   WebKitDOMNode *password_node = NULL;
   guint i, n_elements;
   gboolean found_auth_elements = FALSE;
+  gboolean found_auth_no_username_elements = FALSE;
 
   elements = webkit_dom_html_form_element_get_elements (form);
   n_elements = webkit_dom_html_collection_get_length (elements);
@@ -366,12 +367,13 @@ ephy_web_dom_utils_find_form_auth_elements (WebKitDOMHTMLFormElement *form,
   for (i = 0; i < n_elements; i++) {
     WebKitDOMNode *element;
     char *element_type;
+    char *element_name;
 
     element = webkit_dom_html_collection_item (elements, i);
     if (!WEBKIT_DOM_IS_HTML_INPUT_ELEMENT (element))
       continue;
 
-    g_object_get (element, "type", &element_type, NULL);
+    g_object_get (element, "type", &element_type, "name", &element_name, NULL);
 
     if (g_str_equal (element_type, "text") || g_str_equal (element_type, "email")) {
       /* We found more than one inputs of type text; we won't be saving here. */
@@ -393,13 +395,27 @@ ephy_web_dom_utils_find_form_auth_elements (WebKitDOMHTMLFormElement *form,
 
       password_node = g_object_ref (element);
       found_auth_elements = TRUE;
+
+      /* We found an input that usually doesn't require a separate login
+       * adminpw is used by mailman admin pages */
+      if (g_str_equal (element_name, "adminpw"))
+        found_auth_no_username_elements = TRUE;
     }
 
     g_free (element_type);
+    g_free (element_name);
   }
 
   g_object_unref(elements);
 
+  if (found_auth_no_username_elements && password_node) {
+    g_clear_object (&username_node);
+    *username = NULL;
+    *password = password_node;
+
+    return TRUE;
+  }
+
   if (found_auth_elements && username_node && password_node) {
     *username = username_node;
     *password = password_node;
@@ -414,3 +430,65 @@ ephy_web_dom_utils_find_form_auth_elements (WebKitDOMHTMLFormElement *form,
 
   return FALSE;
 }
+
+/**
+ * ephy_web_dom_utils_get_absolute_position_for_element:
+ * @element: the #WebKitDOMElement.
+ * @x: return address for the x coordinate.
+ * @y: return address for the y coordinate.
+ *
+ * Obtains the coordinate for the top-left of the #WebKitDOMElement, relative
+ * to the origin of the page.
+ **/
+void
+ephy_web_dom_utils_get_absolute_position_for_element (WebKitDOMElement *element,
+                                                      glong *x,
+                                                      glong *y)
+{
+  WebKitDOMElement *parent;
+  long offset_top, offset_left;
+  long parent_x, parent_y;
+
+  g_object_get (element,
+                "offset-left", &offset_left,
+                "offset-top", &offset_top,
+                "offset-parent", &parent,
+                NULL);
+
+  *x = offset_left;
+  *y = offset_top;
+
+  if (!parent)
+    return;
+
+  /* If there's a parent, we keep going. */
+  ephy_web_dom_utils_get_absolute_position_for_element (parent, &parent_x, &parent_y);
+
+  *x += parent_x;
+  *y += parent_y;
+}
+
+/**
+ * ephy_web_dom_utils_get_absolute_bottom_for_element:
+ * @element: the #WebKitDOMElement.
+ * @x: return address for the x coordinate.
+ * @y: return address for the y coordinate.
+ *
+ * Obtains the coordinate for the bottom-left of the #WebKitDOMElement, relative
+ * to the origin of the page.
+ **/
+void
+ephy_web_dom_utils_get_absolute_bottom_for_element (WebKitDOMElement *element,
+                                                    long *x,
+                                                    long *y)
+{
+  long offset_height;
+
+  ephy_web_dom_utils_get_absolute_position_for_element (element, x, y);
+
+  g_object_get (element,
+                "offset-height", &offset_height,
+                NULL);
+
+  *y += offset_height;
+}
